from srsgui import Task
from srsgui import ListInput, InstrumentInput, CommandInput

from srsinst.sr860 import SR860, get_sr860
from srsinst.sr860.instruments.components import Display


class AssignParameterToChannel(Task):
    """
    Assign a parameter to a channel
    """
    InstName = 'inst to control'
    Channel1 = 'channel 1'
    Channel2 = 'channel 2'
    Channel3 = 'channel 3'
    Channel4 = 'channel 4'

    input_parameters = {
        InstName: InstrumentInput(),
        Channel1: CommandInput('display.config[0]', Display.config),
        Channel2: CommandInput('display.config[1]', Display.config),
        Channel3: CommandInput('display.config[2]', Display.config),
        Channel4: CommandInput('display.config[3]', Display.config),
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.lockin = get_sr860(self, self.params[self.InstName])

    def test(self):
        self.logger.info(self.params)

    def cleanup(self):
        pass






import sys
import unittest
from srsinst.sr860 import SR860, Keys

ConnectionString = "vxi11, 172.25.70.129"


class TestSR860(unittest.TestCase):
    lockin = None

    @classmethod
    def setUpClass(cls) -> None:
        args = [arg.strip() for arg in ConnectionString.split(',')]
        print('connection: ', args)
        cls.lockin = SR860(*args)
        cls.lockin.reset()

    @classmethod
    def tearDownClass(cls) -> None:
        cls.lockin.disconnect()

    def setUp(self) -> None:
        self.lockin = TestSR860.lockin
        self.lockin.reset()

    def tearDown(self) -> None:
        pass

    def test_idn(self):
        print(self.lockin.query_text('*idn?'))
        self.assertIn(',SR86', self.lockin.query_text('*idn?'))

    def test_signal_mode(self):
        self.lockin.signal.input_mode = Keys.Current
        self.assertEqual(self.lockin.signal.input_mode, Keys.Current)
        self.lockin.signal.input_mode = Keys.Voltage
        self.assertEqual(self.lockin.signal.input_mode, Keys.Voltage)

    def test_reference_source(self):
        self.lockin.signal.reference_source = Keys.Internal
        self.assertEqual(self.lockin.signal.reference_source, Keys.Internal)
        self.lockin.signal.reference_source = Keys.External
        self.assertEqual(self.lockin.signal.reference_source, Keys.External)

    def test_reference_frequency(self):
        self.lockin.signal.reference_frequency = 1000
        self.assertEqual(self.lockin.signal.reference_frequency, 1000)
        self.lockin.signal.reference_frequency = 10000
        self.assertEqual(self.lockin.signal.reference_frequency, 10000)

    def test_reference_phase(self):
        self.lockin.signal.reference_phase = 0
        self.assertEqual(self.lockin.signal.reference_phase, 0)
        self.lockin.signal.reference_phase = 180
        self.assertEqual(self.lockin.signal.reference_phase, 180)

    def test_reference_harmonic(self):
        self.lockin.signal.reference_harmonic = 1
        self.assertEqual(self.lockin.signal.reference_harmonic, 1)
        self.lockin.signal.reference_harmonic = 2
        self.assertEqual(self.lockin.signal.reference_harmonic, 2)

    def test_reference_trigger_mode(self):
        self.lockin.ref.trigger_mode = Keys.Sine
        self.assertEqual(self.lockin.ref.trigger_mode, Keys.Sine)
        self.lockin.ref.trigger_mode = Keys.PositiveTTL
        self.assertEqual(self.lockin.ref.trigger_mode, Keys.PositiveTTL)

    def test_reference_trigger_input(self):
        self.lockin.ref.trigger_input = Keys.R50Ohms
        self.assertEqual(self.lockin.ref.trigger_input, Keys.R50Ohms)
        self.lockin.ref.trigger_input = Keys.R1Meg
        self.assertEqual(self.lockin.ref.trigger_input, Keys.R1Meg)

    def test_sine_out_amplitude(self):
        self.lockin.sine_out_amplitude = 0.1
        self.assertEqual(self.lockin.sine_out_amplitude, 0.1)
        self.lockin.sine_out_amplitude = 1
        self.assertEqual(self.lockin.sine_out_amplitude, 1)

    def test_sine_out_offset(self):
        self.lockin.sine_out_offset = 0.0
        self.assertEqual(self.lockin.sine_out_offset, 0.0)
        self.lockin.sine_out_offset = 1.0
        self.assertEqual(self.lockin.sine_out_offset, 1.0)


if __name__ == '__main__':
    unittest.main()



import time

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Signal, Reference
from srsinst.sr860.instruments.keys import Keys


class ConfigureSineOutputTask(Task):
    """
Sine output is generated based on the frequency of the selected reference source.
    """
    InstName = 'inst to control'
    SineAmplitude = 'sine amplitude (V)'
    SineOffset = 'sine dc level (V)'
    SineDCMode = 'sine dc mode'

    input_parameters = {
        InstName:  InstrumentInput(),
        SineAmplitude: CommandInput('ref.sine_out_amplitude', Reference.sine_out_amplitude),
        SineOffset: CommandInput('ref.sine_out_offset', Reference.sine_out_offset),
        SineDCMode: CommandInput('ref.sine_out_dc_mode', Reference.sine_out_dc_mode),
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.lockin = get_sr860(self, self.params[self.InstName])

    def test(self):
        amplitude = self.lockin.ref.sine_out_amplitude
        offset = self.lockin.ref.sine_out_offset
        dc_mode = self.lockin.ref.sine_out_dc_mode
        self.logger.info('Sine output amplitude: {:.3f} V, offset: {:.3f} V, dc mode: {}'
                         .format(amplitude, offset, dc_mode))

    def cleanup(self):
        pass




from .sr860 import SR860


class SR865A(SR860):
    _IdString = 'SR865A'
    MaxFrequency = 4e6

    def __init__(self, interface_type=None, *args):
        super().__init__(interface_type, *args)
        self.ref.__class__.internal_frequency.maximum = self.MaxFrequency
        self.ref.__class__.frequency.maximum = self.MaxFrequency




from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Signal, Reference
from srsinst.sr860.instruments.keys import Keys


class SelectTimebaseTask(Task):
    """
The internal timebase provides 25 ppm + 30 uHz accuracy. For better accuracy, \
you can connect an external 10 MHz clock to the timebase input/output connector \
on the rear panel, and select external timebase. Selecting the external source
without an external 10 MHz clock will automatically run with the internal oscillator.
    """

    InstName = 'inst to control'
    TimebaseMode = 'timebase mode'
    TimebaseSource = 'timebase source'

    input_parameters = {
        InstName:       InstrumentInput(),
        TimebaseSource: CommandInput('ref.timebase_source', Reference.timebase_source),
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.lockin = get_sr860(self, self.params[self.InstName])

    def test(self):
        if self.params[self.TimebaseSource] == Keys.External:
            self.lockin.ref.timebase_mode = Keys.Auto
            self.lockin.ref.timebase_source = Keys.External
            self.logger.info('Timebase is set to Auto / External.')
        else:
            self.lockin.ref.timebase_mode = Keys.Internal
            self.lockin.ref.timebase_source = Keys.Internal
            self.logger.info('Timebase is set to the internal oscillator.')

    def cleanup(self):
        pass




from .sr860 import SR860


class SR865(SR860):
    _IdString = 'SR86[A]?,'  # Regular expression including SR865 and SR865A
    MaxFrequency = 2e6

    def __init__(self, interface_type=None, *args):
        super().__init__(interface_type, *args)
        self.ref.__class__.internal_frequency.maximum = self.MaxFrequency
        self.ref.__class__.frequency.maximum = self.MaxFrequency




from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Signal
from srsinst.sr860.instruments.keys import Keys


class SignalToVoltageModeTask(Task):
    """
It sets the input mode to the voltage inout mode. \
Make sure the signal is connected to the voltage input connector A or \
both A and B on the front panel.
    """
    InstName = 'inst to change'
    InputMode = 'input mode'
    InputCoupling = 'input coupling'
    InputShield = 'input shield'
    InputRange = 'input range (V)'
    InputSens = 'input sensitivity (V)'
    TimeConstant = 'time constant (s)'
    FilterSlope = 'filter slope (dB/oct)'
    SyncFilter = 'synchronous filter'
    AdvancedFilter = 'advanced filter'

    input_parameters = {
        InstName:       InstrumentInput(),
        InputMode:      CommandInput('signal.voltage_input_mode', Signal.voltage_input_mode),
        InputCoupling:  CommandInput('signal.voltage_input_coupling', Signal.voltage_input_coupling),
        InputShield:    CommandInput('signal.voltage_input_shield', Signal.voltage_input_shield),
        InputRange:     CommandInput('signal.voltage_input_range', Signal.voltage_input_range),
        InputSens:      CommandInput('signal.voltage_sensitivity', Signal.voltage_sensitivity),
        TimeConstant:   CommandInput('signal.time_constant', Signal.time_constant),
        FilterSlope:    CommandInput('signal.filter_slope', Signal.filter_slope),
        SyncFilter:     CommandInput('signal.sync_filter', Signal.sync_filter),
        AdvancedFilter: CommandInput('signal.advanced_filter', Signal.advanced_filter)
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.lockin = get_sr860(self, self.params[self.InstName])

    def test(self):
        # Change to Voltage mode
        self.lockin.signal.input_mode = Keys.Voltage
        self.logger.info(self.params)
        self.logger.info('Input mode change to the voltage mode')

    def cleanup(self):
        pass




from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput
from srsinst.sr860 import SR860, get_sr860

from srsinst.sr860.instruments.components import Signal
from srsinst.sr860.instruments.keys import Keys


class SignalToCurrentModeTask(Task):
    """
It sets the input mode to the current inout mode. \
Make sure the signal is connected to the current input connector on the front panel.
    """
    InstName = 'inst to change'
    InputGain = 'input gain (Ohm)'
    Sensitivity = 'sensitivity (A)'
    TimeConstant = 'time constant (s)'
    FilterSlope = 'filter slope (dB/oct)'
    SyncFilter = 'synchronous filter'
    AdvancedFilter = 'advanced filter'

    input_parameters = {
        InstName:       InstrumentInput(),
        InputGain:      CommandInput('signal.current_input_gain', Signal.current_input_gain),
        Sensitivity:    CommandInput('signal.current_sensitivity', Signal.current_sensitivity),
        TimeConstant:   CommandInput('signal.time_constant', Signal.time_constant),
        FilterSlope:    CommandInput('signal.filter_slope', Signal.filter_slope),
        SyncFilter:     CommandInput('signal.sync_filter', Signal.sync_filter),
        AdvancedFilter: CommandInput('signal.advanced_filter', Signal.advanced_filter)
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.lockin = get_sr860(self, self.params[self.InstName])

    def test(self):
        self.lockin.signal.input_mode = Keys.Current
        self.logger.info(self.params)
        self.logger.info('Input mode change to the current mode')

    def cleanup(self):
        pass




import time

from srsgui import Task
from srsgui import BoolInput, IntegerListInput, FloatListInput, InstrumentInput, CommandInput

from srsinst.sr860 import get_sr860
from srsinst.sr860.instruments.components import Reference
from srsinst.sr860.instruments.keys import Keys as LKeys

from srsinst.sr542 import get_sr542
from srsinst.sr542.instruments.components import Config
from srsinst.sr542.instruments.keys import Keys as CKeys


class SetRefToExternalWithSR542Task(Task):
    """
Use the internal frequency of SR542 as reference source. \
To use the outer track of a blade, connect the outer track \
reference output connector or inner track for chopping.
Connect
Change the reference source of SR860 to the external and .
    """
    InstName = 'Lock-in to control'
    TriggerMode = 'trigger mode'
    TriggerInput = 'trigger input'
    Phase = 'phase (degree)'
    Harmonic = 'harmonic to detect'
    ChopperName = 'Chopper to control'
    FrequencySource = 'Frequency source'
    ControlTarget = 'control target'
    InternalF = 'internal frequency'
    BladePhase = 'blade phase'

    input_parameters = {
        InstName: InstrumentInput(0),
        Phase: CommandInput('ref.phase', Reference.phase),
        Harmonic: CommandInput('ref.harmonic', Reference.harmonic),

        ChopperName: InstrumentInput(1),
        ControlTarget: CommandInput('config.control_target', Config.control_target),
        InternalF: CommandInput('config.frequency', Config.frequency),
        BladePhase: CommandInput('config.phase', Config.phase),
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.lockin = get_sr860(self, self.params[self.InstName])
        self.chopper = get_sr542(self, self.params[self.ChopperName])

        if self.chopper.config.control_target not in (CKeys.INNER, CKeys.OUTER):
            msg = 'Control target should be either "{}" or "{}, NOT "{}".'\
                .format(CKeys.INNER, CKeys.OUTER, self.params[self.ControlTarget])
            raise ValueError(msg)

    def test(self):
        self.lockin.ref.reference_source = LKeys.External
        self.lockin.ref.trigger_mode = LKeys.PositiveTTL
        self.lockin.ref.trigger_input = LKeys.R1Meg

        initial_motor_state = self.chopper.operate.motor_state

        # Change the reference source to internal
        if self.chopper.operate.motor_state == CKeys.ON and self.chopper.config.source != CKeys.INTERNAL:
            reply = self.ask_question("Is it OK to turn off the chopper \n"
                                      "to change the reference source to internal?")
            if not reply:
                self.logger.info("Aborted not to turn off chopper")
                return
            else:
                self.chopper.operate.motor_state = CKeys.OFF
                time.sleep(1.0)
                self.chopper.config.source = CKeys.INTERNAL

            self.logger.info('Turning back on the chopper')
            self.chopper.operate.motor_state = CKeys.ON
            time.sleep(3.0)

        chopper_f = lockin_f = 0.0
        if self.chopper.operate.motor_state == CKeys.OFF:
            reply = self.ask_question("Is it OK to turn on the chopper \n"
                                      "to check if the frequencies are matched \n"
                                      "between lock-in and the chopper?")
            if reply:
                self.chopper.operate.motor_state = CKeys.ON
                time.sleep(3.0)

            control_target = self.chopper.config.control_target
            chopper_f = self.chopper.operate.frequency_monitor[control_target]
            while chopper_f < 0.98 * self.params[self.InternalF]:
                time.sleep(1.0)
                chopper_f = self.chopper.operate.frequency_monitor[control_target]

            lockin_f = self.lockin.data.value[LKeys.ExternalFrequency]

            self.logger.info('Chopper frequency: {:.3f} Hz Lockin external frequency: {:.3f} Hz'
                             .format(chopper_f, lockin_f))

            valid = 0.95 < chopper_f / lockin_f < 1.05
            if not valid:
                self.ask_question('The external frequency {:3f} not matching with'
                                  'the chopper {} frequency {:.3f}. Check connections'
                                  .format(lockin_f, self.params[self.ControlTarget], chopper_f, None))

        if initial_motor_state == CKeys.OFF:
            reply = self.ask_question("Do you want to turn back off the chopper?")
            if reply:
                self.chopper.operate.motor_state = CKeys.OFF

        phase = self.lockin.ref.phase
        harmonic = self.lockin.ref.harmonic
        trigger_mode = self.lockin.ref.timebase_mode
        trigger_input = self.lockin.ref.trigger_input

        self.logger.info('Lock-in reference source is set to {}.'.format(self.lockin.ref.reference_source))
        self.logger.info('Trigger mode: {}, trigger input: {}'.format(trigger_mode, trigger_input))

        self.logger.info('Phase: {:.3f} degree, harmonic order to detect: {}'
                         .format(phase, harmonic))

    def cleanup(self):
        pass
